package com.example.backend.movies.api;

import java.util.List;

import org.modelmapper.ModelMapper;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.example.backend.categories.service.CategorieService;
import com.example.backend.core.configurations.Constants;
import com.example.backend.favorites.model.FavoriteEntity;
import com.example.backend.favorites.service.FavoriteService;
import com.example.backend.movies.model.MovieEntity;
import com.example.backend.movies.service.MovieService;
import com.example.backend.users.service.UserService;

import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.security.core.Authentication;

import jakarta.validation.Valid;

import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.PostMapping;

@Controller
@RequestMapping(MovieUserController.URL)
public class MovieUserController {

    public static final String URL = "/movies";
    private static final String MOVIE_VIEW = "movies";
    private static final String MOVIE_EDIT_VIEW = "movie-edit";
    private static final String PAGE_ATTRIBUTE = "page";
    private static final String MOVIE_ATTRIBUTE = "movie";
    private static final String CATEGORIEID_ATTRIBUTE = "categorieId";

    private final MovieService movieService;
    private final CategorieService categorieService;
    private final UserService userService;
    private final FavoriteService favoriteService;

    private final ModelMapper modelMapper;

    public MovieUserController(MovieService movieService,
            CategorieService categorieService,
            ModelMapper modelMapper,
            UserService userService,
            FavoriteService favoriteService) {
        this.modelMapper = modelMapper;
        this.categorieService = categorieService;
        this.movieService = movieService;
        this.userService = userService;
        this.favoriteService = favoriteService;
    }

    private MovieDTO toDto(MovieEntity entity) {
        return modelMapper.map(entity, MovieDTO.class);
    }

    private MovieEntity toEntity(MovieDTO dto) {
        final MovieEntity entity = modelMapper.map(dto, MovieEntity.class);
        entity.setCategorie(categorieService.get(dto.getCategorieId()));
        return entity;
    }

    @GetMapping()
    public String getAll(@RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        model.addAttribute("movies",
                movieService.getAll(0, page, Constants.DEFUALT_PAGE_SIZE)
                        .stream()
                        .map(this::toDto)
                        .toList());
        model.addAttribute(CATEGORIEID_ATTRIBUTE, model);
        model.addAttribute("categories", categorieService.getAll());

        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        Integer currentUserId = userService.getCurrentUserId(authentication.getName());

        List<FavoriteEntity> favs = favoriteService.getAll(currentUserId);

        model.addAttribute("favorites", favs);
        model.addAttribute(CATEGORIEID_ATTRIBUTE, 0);
        return MOVIE_VIEW;
    }

    @GetMapping("/categorieId/{id}")
    public String getAll(@PathVariable(name = "id") Integer categorieId,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        model.addAttribute("movies",
                movieService.getAll(categorieId, page, Constants.DEFUALT_PAGE_SIZE)
                        .stream()
                        .map(this::toDto)
                        .toList());

        model.addAttribute("categories", categorieService.getAll());

        Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
        Integer currentUserId = userService.getCurrentUserId(authentication.getName());

        List<FavoriteEntity> favs = favoriteService.getAll(currentUserId);

        model.addAttribute("favorites", favs);
        model.addAttribute(PAGE_ATTRIBUTE, page);
        model.addAttribute(CATEGORIEID_ATTRIBUTE, categorieId);
        return MOVIE_VIEW;
    }

    @GetMapping("/addToFavorite/{id}")
    public String changeStatusForFavorite() {
        return Constants.REDIRECT_VIEW + URL;
    }

    @GetMapping("/countView")
    public Integer countView(@RequestParam(name = "movieId", defaultValue = "0") Integer movieId) {
        return movieService.countView(movieId);
    }
}
